// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.gui;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.parser.*;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;


public class SingleProver extends Main {

    /** contains a list of all proofs */
    private JScrollPane proofListView;
    
    
    private ProofList proofList;
    

    /** creates the main frame of the prover. It takes as argument an array of
     * terms that have to be proven to be correct. 
     */
    public SingleProver(String title, ProofSettings settings) {
	
	super("KeY -- Proving "+title, settings);
	Config.DEFAULT.setDefaultFonts();

	
        KeYMediator medi = new KeYMediator(this);
        setMediator(medi);
        proofList = new ProofList(medi);


	ToolTipManager.sharedInstance().setDismissDelay(20000);
	ToolTipManager.sharedInstance().setInitialDelay(100);
        
        proofListView = new JScrollPane();	
        addTab("Proofobligations", proofListView, "Currently loaded proofs");

	setVisible(true);
    }

    
    protected Proof setUpNewProof(String problemID,
				  Term problem,
				  TacletIndex rules,
				  Namespace variables,
				  Namespace functions,
				  Namespace sorts,
				  Namespace heuristics,
				  JavaInfo javaInfo) {
	Proof p=super.setUpNewProof(problemID, problem, rules, variables, functions, 
				    sorts, heuristics, javaInfo);
	addToProofList(p);
	return p;
    }


    protected void addToProofList(Proof proof) {
        proofList.addProof(proof);	
	
	// GUI 
	proofList.setSize(proofList.getPreferredSize());
	proofListView.setViewportView(proofList);
    }

			    
    public void updateUI() {
	super.updateUI();
    	if (proofListView != null) {
	    proofListView.updateUI();
	}
    }


    /** exit  */
    protected void exitMain() {
	giveUp();
    }

    public void giveUp() {
	mediator.fireShutDown(new GUIEvent(this));
    }
}
