// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic;

import junit.framework.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.LogicVariable;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.*;

/** class tests the term factory
*/


public class TestTerm extends TestCase { 


    private TermFactory tf=TermFactory.DEFAULT;

    private Term et1;
    private Sort sort1=new PrimitiveSort(new Name("S1"));
    private Sort sort2=new PrimitiveSort(new Name("S2"));
    private Sort sort3=new PrimitiveSort(new Name("S3"));
    private String invalidTermMessage="Term not valid with given subterms";
    	

    Function p=new Function(new Name("p"),Sort.FORMULA,new Sort[]{sort1});  
        //p(:S1):BOOL
    LogicVariable x=new LogicVariable(new Name("x"),sort1);  //x:S1
    Function q=new Function(new Name("q"),Sort.FORMULA,
			    new Sort[]{new PrimitiveSort(new Name("Whatever"))}); 
        //q(:Whatever):BOOL
    LogicVariable z=new LogicVariable(new Name("z"),sort1); //z:S1
    Function r=new Function(new Name("r"),Sort.FORMULA,new Sort[]{sort1, sort2});
        //r(:S1, :S2):BOOL
    LogicVariable y=new LogicVariable(new Name("y"),sort3); //y:S3
    LogicVariable w=new LogicVariable(new Name("w"),sort2); //w:S2
    Function f=new Function(new Name("f"),sort1, new Sort[]{sort3}); 
        // f(:S3):S1



    public TestTerm(String name) {
	super(name);
    }

    public void setUp() {

	Term et_x=new OpTerm(x,new Term[0]);
	Term et_px=new OpTerm(p, new Term[]{et_x});
	et1=et_px;

    }



    private Term t1(){
	Term t_x=tf.createFunctionTerm(x, new Term[0]);
	Term t_px=tf.createFunctionTerm(p, new Term[]{t_x});
	return t_px;
    }
  
    private Term t2(){
	Term t_x=tf.createFunctionTerm(x, new Term[]{});
	Term t_w=tf.createFunctionTerm(w, new Term[]{});
	return tf.createFunctionTerm(r, new Term[]{t_x,t_w});
    }

    private Term t3() {
	Term t_y=tf.createFunctionTerm(y, new Term[]{});
	return tf.createFunctionTerm(f, new Term[]{t_y});
    }

    public void testFreeVars1() {
	Term t_allxt2=tf.createQuantifierTerm(Op.ALL,new LogicVariable[]{x},t2());
	Term t_allxt2_andt1=tf.createJunctorTerm(Op.AND,t_allxt2,t1());
	assert(t_allxt2_andt1.freeVars().contains(w) 
	       && t_allxt2_andt1.freeVars().contains(x));
    }

   public void testFreeVars2() {
	Term t_allxt2=tf.createQuantifierTerm(Op.ALL,new LogicVariable[]{w},t2());
	Term t_allxt2_andt1=tf.createJunctorTerm(Op.AND,t_allxt2,t1());
	assert(!t_allxt2_andt1.freeVars().contains(w) 
	       && t_allxt2_andt1.freeVars().contains(x));
    }
    
    public void testFreeVars3() {
	Term t_allxt1=tf.createQuantifierTerm(Op.ALL,new LogicVariable[]{x},t2());
	Term t_allxt1_andt2=tf.createJunctorTerm(Op.AND,t_allxt1,t1());
	Term t_exw_allxt1_andt2=tf.createQuantifierTerm(Op.EX, 
							new LogicVariable[]{w}, 
							t_allxt1_andt2); 
	assert(!t_exw_allxt1_andt2.freeVars().contains(w) 
	       && t_exw_allxt1_andt2.freeVars().contains(x));
    }

   public void testFreeVars4() {
	Term t_allxt1=tf.createQuantifierTerm(Op.ALL,new LogicVariable[]{x},t2());
	Term t_allxt1_andt2=tf.createJunctorTerm(Op.AND,t_allxt1,t1());
	Term t_exw_allxt1_andt2=tf.createQuantifierTerm(Op.EX, 
							new LogicVariable[]{w,x}, 
							t_allxt1_andt2); 
	assert(!t_exw_allxt1_andt2.freeVars().contains(w) 
	       && !t_exw_allxt1_andt2.freeVars().contains(x));
    }


    public void testProgramElementEquals() {

	Term match1 = TacletForTests.parseTerm("<{ int i = 0; }>true ");
	Term match2 = TacletForTests.parseTerm("<{ int i = 0; }>true ");
	assert("Terms should be equal.", match1.equals(match2));

	Term match3 = TacletForTests.parseTerm("<{ int j = 0; }>true ");
	assert("Terms should not be equal.", !match1.equals(match3));

    }
   

}
