// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
public class Epsilon extends Op {
    
    /** creates an epsilon operator */
    public Epsilon(Name name) {
	super(name);
    }


    /**
     * @return sort of the bound variable for the 0th subterm or
     * throws an IllegalArgumentException if given term not valid.
     */
     public Sort sort(Term term) {
	 if (validTopLevel(term))
	    return term.varsBoundHere(0).getQuantifiableVariable(0).sort();
	else throw new IllegalArgumentException("Cannot determine sort of "+
						"invalid term (Wrong arity or"+
						" more than one or no"+
						"variable bound).");
    }  

    /**
     * @return true iff arity of the term is 1 and there is 
     * exactly one var bound.
     */
    public boolean validTopLevel(Term term) {
	return (term.arity()==1 && term.varsBoundHere(0).size()==1);
    }


   /** @return arity of the Epsilon operator as int */
    public int arity() {
	return 1;
    }

}
