// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001 Universitaet Karlsruhe, Germany
//                    and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General 
// Public License. See LICENSE.TXT for details.
//
package de.uka.ilkd.key.logic.sort;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.Type;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.util.ExtList;
import java.util.Iterator;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.java.visitor.JavaIterator;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.ProgramMethod;

public class ProgramSVSort extends de.uka.ilkd.key.logic.sort.PrimitiveSort {

    public static final ProgramSVSort STATEMENT
	= new StatementSort();

    public static final ProgramSVSort EXPRESSION
	= new ProgramSVSort(new Name("Expression"));

    public static final ProgramSVSort SIMPLEEXPRESSION 
	= new SimpleExpressionSort();

     public static final ProgramSVSort SIMPLEJAVABOOLEXPRESSION 
 	= new SimpleExpressionSpecialTypeSort
 	("SimpleExpressionOfTypeBoolean", new
 	    PrimitiveType[]{PrimitiveType.JAVA_BOOLEAN});

    public static final ProgramSVSort SIMPLEJAVABYTEEXPRESSION 
	= new SimpleExpressionSpecialTypeSort
	("SimpleExpressionOfTypeByte", new
	    PrimitiveType[]{PrimitiveType.JAVA_BYTE});

    public static final ProgramSVSort SIMPLEJAVASHORTEXPRESSION 
	= new SimpleExpressionSpecialTypeSort
	("SimpleExpressionOfTypeShort", new
	    PrimitiveType[]{PrimitiveType.JAVA_SHORT});

    public static final ProgramSVSort SIMPLEJAVAINTEXPRESSION 
	= new SimpleExpressionSpecialTypeSort
	("SimpleExpressionOfTypeInt", new
	    PrimitiveType[]{PrimitiveType.JAVA_INT});

    public static final ProgramSVSort SIMPLEJAVALONGEXPRESSION 
	= new SimpleExpressionSpecialTypeSort
	("SimpleExpressionOfTypeLong", new
	    PrimitiveType[]{PrimitiveType.JAVA_LONG});

    public static final ProgramSVSort SIMPLEJAVABYTESHORTINTEXPRESSION
	= new SimpleExpressionSpecialTypeSort
	("SimpleExpressionOfTypeLong", new
	    PrimitiveType[]{PrimitiveType.JAVA_BYTE,
			     PrimitiveType.JAVA_SHORT,PrimitiveType.JAVA_INT});

    public static final ProgramSVSort SIMPLEANYJAVATYPEEXPRESSION 
	= new SimpleExpressionSpecialTypeSort
	("SimpleExpressionOfTypeLong", new
	    PrimitiveType[]{PrimitiveType.JAVA_BYTE,
			     PrimitiveType.JAVA_SHORT,PrimitiveType.JAVA_INT,
			     PrimitiveType.JAVA_LONG});

    public static final ProgramSVSort NONSIMPLEEXPRESSION 
	= new NonSimpleExpressionSort();

    public static final ProgramSVSort BOOLEANEXPRESSION
	= new BooleanExpressionSort();

    public static final ProgramSVSort NONSIMPLEBOOLEANEXPRESSION
	= new NonSimpleBooleanExpressionSort();

    public static final ProgramSVSort LEFTHANDSIDE = new LeftHandSideSort();

    public static final ProgramSVSort SIMPLELEFTHANDSIDE 
	= new SimpleLeftHandSideSort();

    public static final ProgramSVSort FORINIT 
	= new ProgramSVSort(new Name("ForInit"));

    public static final ProgramSVSort STATEMENTEXPRESSIONLIST
	= new ProgramSVSort(new Name("StatementExpressionList"));

    public static final ProgramSVSort STATEMENTLIST
	= new StatementListSort();

    public static final ProgramSVSort EXPRESSIONLIST
	= new ExpressionListSort();


    public static final ProgramSVSort LABEL
	= new ProgramSVSort(new Name("Label"));

    public static final ProgramSVSort TYPE
	= new ProgramSVSort(new Name("Type"));

    public static final ProgramSVSort VARIABLE
	= new ProgramVariableSort();

    public static final ProgramSVSort BOOLEANVARIABLE
	= new ProgramSVSort(new Name("BooleanVariable"));

    public static final ProgramSVSort CATCHFINALLY
	= new CatchFinallySort();

    public static final ProgramSVSort VARIABLEINIT
	= new ProgramSVSort(new Name("VariableInitializer"));

    public static final ProgramSVSort NEW = new NewSVSort();

    public static final ProgramSVSort METHODREFERENCE
	= new MethodReferenceSort();

    public static final ProgramSVSort METHODBODY
	= new MethodBodySort();

    public static final ProgramSVSort BRANCHLIST
	= new BranchListSort();

    public ProgramSVSort(Name name) {
	super(name);
    }

    public boolean canStandFor(Term t) {
	return false;
    }

    protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	return true;
    }

    public boolean canStandFor(ProgramElement check, 
			       JavaInfo javaInfo) {
	JavaIterator it = new JavaIterator(check);
	ProgramElement pe = it.element();
	while (it.hasNext()) {
	    if (!allowed(pe, javaInfo)) {
		return false;
	    }
	    pe = it.next();
	}
	return true;
    }

    static boolean methodConstrReference(ProgramElement pe) {
	    return ((pe instanceof MethodReference)
		|| (pe instanceof ConstructorReference)
		|| (pe instanceof SpecialConstructorReference));
    }

    public ProgramElement getSVWithSort(ExtList l, Class alternative) {	
	Iterator it=l.iterator();
	while (it.hasNext()) {
	    Object o=it.next();
	    if (o instanceof SchemaVariable
		&& (((SchemaVariable)o).sort()==this)) {
		    return (ProgramElement) o;
	    }
	    else if ((alternative.isInstance(o)) 
		     && (! (o instanceof SchemaVariable))) {
		return (ProgramElement) o;
	    }
	}
	return null;
    }

    static private class SimpleLeftHandSideSort extends ProgramSVSort{

	public SimpleLeftHandSideSort() {
	    super(new Name("SimpleLeftHandSide"));
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof de.uka.ilkd.key.java.expression.Operator) {
		return false;
	    }
	    if (pe instanceof ParenthesizedExpression) {
		return false;
	    }
	    if (pe instanceof Literal) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}
    }


    static private class LeftHandSideSort extends ProgramSVSort{

	public LeftHandSideSort() {
	    super(new Name("LeftHandSide"));
	}

	public boolean canStandFor(ProgramElement pe, 
				   JavaInfo javaInfo) {
	    if (methodConstrReference(pe)) {
		return false;  //last component of access path must be without sideeffects
	    }
	    return super.canStandFor(pe, javaInfo);
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return (!(pe instanceof de.uka.ilkd.key.java.expression.Operator));
	}
    }

    static private class MethodReferenceSort extends ProgramSVSort{

	public MethodReferenceSort() {
	    super(new Name("MethodReference"));
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (pe instanceof MethodReference) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(Term t) {
	    return (t.op() instanceof ProgramMethod);
	}
    }


    static private class ProgramVariableSort extends ProgramSVSort{

	public ProgramVariableSort() {
	    super(new Name("ProgramVariable"));
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (pe instanceof ProgramVariable
		|| pe instanceof VariableSpecification
		|| pe instanceof VariableReference) {
		return true;
	    }
	    return false;
	}
    }


    static private class SimpleExpressionSort extends ProgramSVSort{

	public SimpleExpressionSort() {
	    super(new Name("SimpleExpression"));
	}

	protected SimpleExpressionSort(String name) {
	    super(new Name(name));
	}

	protected boolean allowed(ProgramElement pe,
				  JavaInfo javaInfo) {
	    if (pe instanceof Negative) {
 		return ((Negative)pe).getChildAt(0) instanceof Literal;
 	    }	   
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof de.uka.ilkd.key.java.expression.Operator) {
		return false;
	    }
	    if (pe instanceof ParenthesizedExpression) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    if (pe instanceof TypeCast) {
		return false;
	    }
	    if (pe instanceof NewArray) {
		return false;
	    }
	    if (pe instanceof LogicalAnd) {
		return false;
	    }
	    if (pe instanceof LogicalOr) {
		return false;
	    }
	    if (pe instanceof Equals) {
		return false;
	    }
	    if (pe instanceof NotEquals) {
 		return false;
            }
	    if (pe instanceof LessThan) {
 		return false;
            }
	    if (pe instanceof LessOrEquals) {
 		return false;
            }
	    if (pe instanceof GreaterThan) {
 		return false;
            }
	    if (pe instanceof GreaterOrEquals) {
 		return false;
            }
// 	    if (pe instanceof de.uka.ilkd.key.java.expression.Operator
// 		&& !(pe instanceof 
// 		     de.uka.ilkd.key.java.expression.operator.Equals)) {
// 		return false;
// 	    }
	    return !methodConstrReference(pe);
	}

	public boolean canStandFor(Term t) {
	    return true; //%%%%%%%%
	}

    }

    static private class SimpleExpressionSpecialTypeSort 
	extends SimpleExpressionSort{

	private PrimitiveType[] allowed_types;

	public SimpleExpressionSpecialTypeSort(String name, PrimitiveType[] allowed_types) {
	    super(name);
	    this.allowed_types = allowed_types;
	}


	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    if (!super.allowed(check, javaInfo)) {
		return false;
	    }
	    KeYJavaType kjt =
		javaInfo.getPrimitiveKeYJavaType((Expression)check);
	    for (int i=0; i<allowed_types.length; i++){
		if (kjt.getJavaType() == allowed_types[i])
		    return true;
	    }
	    return false;
	}
    }


    static private class NonSimpleExpressionSort extends ProgramSVSort{

	public NonSimpleExpressionSort() {
	    super(new Name("NonSimpleExpression"));
	}

	private boolean checkArrayPrefix(ReferencePrefix prefix) {
	    if (prefix == null || 
		prefix instanceof NewArray ||
		prefix instanceof MethodReference) {	    
		return true;
	    } else {
		return false;
	    }
	}

	private boolean checkDimensionExpressions
	    (ArrayOfExpression dims, JavaInfo javaInfo) {
	    for (int i = 0; i < dims.size(); i++) {
		if(canStandFor(dims.getExpression(i), javaInfo)) {
		    return true;
		}
	    }
	    return false;
	} 

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return true;
	    }
	    if (pe instanceof ParenthesizedExpression) {
		return true;
	    }
	    if (pe instanceof Instanceof) {
		return true;
	    }
	    if (pe instanceof TypeCast) {
		return true;
	    }
	    if (pe instanceof Equals) {
 		return true;
            }
	    if (pe instanceof NotEquals) {
 		return true;
            }
	    if (pe instanceof LessThan) {
 		return true;
            }
	    if (pe instanceof LessOrEquals) {
 		return true;
            }
	    if (pe instanceof GreaterThan) {
 		return true;
            }
	    if (pe instanceof GreaterOrEquals) {
 		return true;
            }
	    if (pe instanceof de.uka.ilkd.key.java.expression.Operator) {
 		return true;
            }
	    if (pe instanceof ArrayReference) {
		ArrayReference array = (ArrayReference) pe;				
		return checkArrayPrefix(array.getReferencePrefix()) &&
		    checkDimensionExpressions(array.getDimensionExpressions(),
					      javaInfo);
	    }
	    if ((pe instanceof MethodReference)
		|| (pe instanceof ConstructorReference)
		|| (pe instanceof SpecialConstructorReference)) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    JavaIterator it = new JavaIterator(check);
	    ProgramElement pe = it.element();
	    while (it.hasNext()) {
		if (allowed(pe, javaInfo)) {
		    return true;
		}
		pe = it.next();
	    }
	    return false;
	}


    }


    static private class BooleanExpressionSort extends ProgramSVSort{

	public BooleanExpressionSort() {
	    super(new Name("BooleanExpression"));
	}

	private boolean checkArrayPrefix(ReferencePrefix prefix) {
	    if (prefix == null || 
		prefix instanceof NewArray ||
		prefix instanceof MethodReference) {	    
		return true;
	    } else {
		return false;
	    }
	}

	private boolean checkDimensionExpressions
	    (ArrayOfExpression dims, JavaInfo javaInfo) {
	    for (int i = 0; i < dims.size(); i++) {
		if(canStandFor(dims.getExpression(i), javaInfo)) {
		    return true;
		}
	    }
	    return false;
	} 

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof ParenthesizedExpression) {
		return true;
	    }
	    if (pe instanceof BooleanLiteral) {
		return true;
	    }
	    if (pe instanceof ProgramVariable) {
		return true;
	    }
	    if (pe instanceof Instanceof) {
		return true;
	    }
	    if (pe instanceof Equals) {
 		return true;
            }
	    if (pe instanceof NotEquals) {
 		return true;
            }
	    if (pe instanceof LessThan) {
 		return true;
            }
	    if (pe instanceof LessOrEquals) {
 		return true;
            }
	    if (pe instanceof GreaterThan) {
 		return true;
            }
	    if (pe instanceof GreaterOrEquals) {
 		return true;
            }
	    if (pe instanceof ArrayReference) {
		ArrayReference array = (ArrayReference) pe;				
		return checkArrayPrefix(array.getReferencePrefix()) &&
		    checkDimensionExpressions(array.getDimensionExpressions(),
					      javaInfo);
	    }
	    if ((pe instanceof MethodReference)
		|| (pe instanceof ConstructorReference)
		|| (pe instanceof SpecialConstructorReference)) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    KeYJavaType kjt =
		javaInfo.getPrimitiveKeYJavaType((Expression)check);
	    if (kjt.getJavaType() != PrimitiveType.JAVA_BOOLEAN)
		return false;
	    JavaIterator it = new JavaIterator(check);
	    ProgramElement pe = it.element();
	    while (it.hasNext()) {
		if (allowed(pe, javaInfo)) {
		    return true;
		}
		pe = it.next();
	    }
	    return false;
	}


    }

    static private class NonSimpleBooleanExpressionSort extends ProgramSVSort{

	public NonSimpleBooleanExpressionSort() {
	    super(new Name("NonSimpleBooleanExpression"));
	}

	private boolean checkArrayPrefix(ReferencePrefix prefix) {
	    if (prefix == null || 
		prefix instanceof NewArray ||
		prefix instanceof MethodReference) {	    
		return true;
	    } else {
		return false;
	    }
	}

	private boolean checkDimensionExpressions
	    (ArrayOfExpression dims, JavaInfo javaInfo) {
	    for (int i = 0; i < dims.size(); i++) {
		if(canStandFor(dims.getExpression(i), javaInfo)) {
		    return true;
		}
	    }
	    return false;
	} 

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof ParenthesizedExpression) {
		return true;
	    }
	    if (pe instanceof Instanceof) {
		return true;
	    }
	    if (pe instanceof Equals) {
 		return true;
            }
	    if (pe instanceof NotEquals) {
 		return true;
            }
	    if (pe instanceof LessThan) {
 		return true;
            }
	    if (pe instanceof LessOrEquals) {
 		return true;
            }
	    if (pe instanceof GreaterThan) {
 		return true;
            }
	    if (pe instanceof GreaterOrEquals) {
 		return true;
            }
 	    if (pe instanceof LogicalAnd) {
 		return true;
            }
 	    if (pe instanceof LogicalOr) {
 		return true;
            }
	    if (pe instanceof ArrayReference) {
		ArrayReference array = (ArrayReference) pe;				
		return checkArrayPrefix(array.getReferencePrefix()) &&
		    checkDimensionExpressions(array.getDimensionExpressions(),
					      javaInfo);
	    }
	    if ((pe instanceof MethodReference)
		|| (pe instanceof ConstructorReference)
		|| (pe instanceof SpecialConstructorReference)) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    KeYJavaType kjt =
		javaInfo.getPrimitiveKeYJavaType((Expression)check);
	    if (kjt.getJavaType() != PrimitiveType.JAVA_BOOLEAN)
		return false;
	    JavaIterator it = new JavaIterator(check);
	    ProgramElement pe = it.element();
	    while (it.hasNext()) {
		if (allowed(pe, javaInfo)) {
		    return true;
		}
		pe = it.next();
	    }
	    return false;
	}


    }
 

    static private class JavaTypeByteSort extends ProgramSVSort{

	public JavaTypeByteSort() {
	    super(new Name("JavaTypeByte"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }

    static private class JavaTypeShortSort extends ProgramSVSort{

	public JavaTypeShortSort() {
	    super(new Name("JavaTypeShort"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }

    static private class JavaTypeIntSort extends ProgramSVSort{

	public JavaTypeIntSort() {
	    super(new Name("JavaTypeInt"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }

    static private class JavaTypeLongSort extends ProgramSVSort{

	public JavaTypeLongSort() {
	    super(new Name("JavaTypeLong"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }


    static private class IntegerLiteralSort extends ProgramSVSort{

	public IntegerLiteralSort() {
	    super(new Name("IntegerLiteral"));
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof IntLiteral) {  

		return true;
	    }
	    return false;
	}

    }

    static private class LongLiteralSort extends ProgramSVSort{

	public LongLiteralSort() {
	    super(new Name("LongLiteral"));
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof LongLiteral) {

		return true;
	    }
	    return false;
	}

    }


    static private class CatchFinallySort extends ProgramSVSort{

	public CatchFinallySort() {
	    super(new Name("CatchFinally"));
	}

	
	public boolean canStandFor(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Catch || pe instanceof Finally) {
		return true;
	    } 
	    return false;
	}
    }

    static private class StatementSort extends ProgramSVSort{

	public StatementSort() {
	    super(new Name("Statement"));
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Catch) {
		return false;
	    }
	    return true;
	}
    }


    static private class NewSVSort extends ProgramSVSort{

	public NewSVSort() {
	    super(new Name("New"));
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    if (check instanceof New) {
		return true;
	    }
	    return false;
	}
    }


    static private class StatementListSort extends ProgramSVSort{

	public StatementListSort() {
	    super(new Name("StatementList"));
	}


	public boolean canStandFor(ProgramElement pe,
				  JavaInfo javaInfo) {
	    if (pe instanceof Statement) {
		return true;
	    }
	    return false;
	}
    }

    static private class ExpressionListSort extends ProgramSVSort{

	public ExpressionListSort() {
	    super(new Name("ExpressionList"));
	}


	public boolean canStandFor(ProgramElement pe,
				  JavaInfo javaInfo) {
	    if (pe instanceof Expression) {
		return true;
	    }
	    return false;
	}
    }

    static private class MethodBodySort extends ProgramSVSort{

	public MethodBodySort() {
	    super(new Name("MethodBody"));
	}

	public boolean canStandFor(ProgramElement check, JavaInfo javaInfo) {
	    return allowed(check, javaInfo);
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return (pe instanceof MethodBodyStatement);
	}
    }

    static private class BranchListSort extends ProgramSVSort{

	public BranchListSort() {
	    super(new Name("BranchList"));
	}


	public boolean canStandFor(ProgramElement pe,
				  JavaInfo javaInfo) {
	    if (pe instanceof Branch) {
		return true;
	    }
	    return false;
	}
    }


}
