// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001 Universitaet Karlsruhe, Germany
//                    and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General 
// Public License. See LICENSE.TXT for details.
//


package de.uka.ilkd.key.ocl.tudresden;

// attention: do not import java.util.Collection
// The tudresden-package contains a class with the same name
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.LinkedList;

import tudresden.ocl.check.types.*;
import tudresden.ocl.check.OclTypeException;
import com.togethersoft.openapi.sci.*;
import com.togethersoft.openapi.sci.enum.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.rwi.enum.*;

import recoder.service.*;
import recoder.abstraction.Package;
import recoder.list.*;
import recoder.convenience.*;
import recoder.io.*;
import recoder.util.*;
import recoder.java.*;

/**
* This class correspondeds to the OCL type ANY and holds references to the
* features the type it represents.
*/
public class TJClassifier implements Any {

    String name, qualifiedName;
    HashMap features, assocEnds;
    HashMap recoderclasses;
    
    TJClassifier(String name, String fullName, HashMap rc) {
	this.name=name;
	this.qualifiedName=fullName;
	features=new HashMap();
	assocEnds=new HashMap();
        recoderclasses = rc;
    }
    
    public String getName() {
	return name;
    }

    public String getQualifiedName() {
	return qualifiedName;
    }

    public HashMap getFeatureMap(){
	return features;
    }
    
    public HashMap getAssocMap() {
    	return assocEnds;
    }

    public String getInvariant(){
	SciModel model = SciModelAccess.getModel(); 
	
	SciClass cls = model.findClass
	    (SciLanguage.JAVA, getQualifiedName());
	String result = cls.getTagList().
	    getTagValue("invariants");
	if (result!=null)
	    return result;
	else
	    return "";
    }

    public void setInvariant(String arg){
	SciModel model = SciModelAccess.getModel(); 
	
	SciClass cls = model.findClass
	    (SciLanguage.JAVA, getQualifiedName());
	cls.getTagList().setTagValue("invariants", arg);
    }

    
    public TJFeature getFeature(String feature) {
	//	System.out.println("  getFeature aufgerufen fr: "+feature);
	return (TJFeature) features.get(feature);
    }
    
    /** @return true if an instance of the type represented by the called object
     *               can replace an instance of the type given as parameter
     */
    public boolean conformsTo(Type t) {
	//	System.out.println(" conformsTo aufgerufen, Parameter: ");
        //System.out.println("    t: " + t.getClass().getName() + " : " + t + ", and this is: " + this.getClass().getName() + " : " + this );
        if (t instanceof TJClassifier) {
	    //	    System.out.println("    " + t + " is TJClassifier");
	    return conformsTo((recoder.java.declaration.TypeDeclaration)(recoderclasses.get(((TJClassifier)t).getName())));
        } else {
	    //            System.out.println("    " + t + " is no TJClassifier");
            return false;
        }
    }
    

    protected boolean conformsTo(recoder.java.declaration.TypeDeclaration t) {
	//	 	System.out.println(" conformsTo called, Parameter: ");
	//        System.out.println("    t: " + t.getClass().getName() + " :: " + t + " and this is: " + this.getClass().getName() + " :: " + this );
        ClassTypeList supertypes = ((recoder.java.declaration.TypeDeclaration)recoderclasses.get(this.getName())).getAllSupertypes();
        for (int i = 0; i < supertypes.size(); i++) {
            try {
		if (((recoder.java.declaration.TypeDeclaration)supertypes.getClassType(i)).equals(t)) {
                    return true;
                }
            } catch (Exception e) {
                System.out.println("    Ooops, no TypeDeclaration: " + e);
	    }
        }
	return false;
    }
    
    public TJBehaviouralFeature getBehavioralFeature(String name, Type[] params) {
	//	System.out.print("  getBehaviouralFeature aufgerufen fr: "+name);
	//		for (int i=0; i<params.length; i++)
	//			System.out.println(", "+params[i]);
	return (TJBehaviouralFeature) features.get(name);
    }
    
    public TJStructuralFeature getStructuralFeature(String name) {
	//	System.out.println("  getStructuralFeature aufgerufen fr: "+name);
	return (TJStructuralFeature) features.get(name);
    }
    
    public TJAssocEnd getAssociationEnd(String assocEnd) {
	// 	System.out.println("  getAssociationEnd aufgerufen");
	return (TJAssocEnd) assocEnds.get(assocEnd);
    }
    
    /**
     * @returns an Array with the supertypes 
     */
   public String[] getSupertypes() {
   //System.out.println("getSupertypes aufgerufen");
       Iterator it = recoderclasses.keySet().iterator();
       System.out.print("Keys of recoderclasses: ");
       while (it.hasNext())
       System.out.print(it.next()+ ", ");
       System.out.println();
 
       it = recoderclasses.values().iterator();
       System.out.print("Values of recoderclasses: ");
       while (it.hasNext())
       System.out.print(it.next()+ ", ");
       System.out.println();
       
       recoder.java.declaration.TypeDeclaration ratThis =
   (recoder.java.declaration.TypeDeclaration)
   (recoderclasses.get(this.getName()));
   ClassTypeList ctl = ratThis.getAllSupertypes();
   String[] result = new String[ctl.size()];
   for (int l = 0; l < ctl.size(); l++) {
       String pack = ctl.getClassType(l).getPackage().getName();
       if (pack.equals("")){
       System.out.println("  --- "+ctl.getClassType(l).getName());
       result[l] = ((ctl.getClassType(l).getName()));
       }
       else {
       System.out.println("  --- "+pack+"."+ctl.getClassType(l).getName());
       result[l] = (
           (ctl.getClassType(l).getPackage().getName()
            + "." +ctl.getClassType(l).getName()));
       }
   }
     return result;
   }
    
    /** navigate to the association end or attribute <i>name</i>, possibly with
     *  qualifier types; unnamed association ends must by made available by
     *  implementing methods with the name of the association end's type, but
     *  beginning with a lower case character (see OCL specification for more
     *  details)
     *
     *  @param qualifiers the qualifier types; may (and will in most cases) be
     *                    <CODE>null</CODE>, but never an array with length 0
     */
    public Type navigateQualified(String name, Type[] qualifiers) {
	//	System.out.println("  navigateQualified aufgerufen. " + "Name: "+name+", Qualifieres:" );
	if (qualifiers!=null){
	    for (int i=0; i<qualifiers.length; i++)
		System.out.println(qualifiers[i]+", ");
	}
	if (qualifiers!=null && qualifiers.length>0) {
	    throw new OclTypeException(
				       "feature \""+name+"\" of type "+this+" cannot be accessed with qualifier"
				       );
	}
	Type ret=null;
	ret=Basic.navigateAnyQualified(name, this, qualifiers);
	if (ret!=null) return ret;
	if (getFeature(name)!=null)
	    ret=getFeature(name).getType();
	else if (getAssociationEnd(name)!=null) {
	    ret=getAssociationEnd(name).getType();
	    if (getAssociationEnd(name).getMultiplicity().getMax()!=1) {
		// add distincion between ordered and unordered assocs. here
		ret=new Collection(Collection.SET, ret);
	    }
	}
	if (ret==null) {
	    throw new OclTypeException(this.name+" has no feature "+name);
	}
	return ret;
    }
    
    /**
     * Navigate to the result type of the operation <i>name</i>. This must return
     * a valid type whether the specified operation is a query or not.
     * @returns the type of a method with the string name and the type-array params
     * @param name the name of the wanted feature
     * @param params an array of OCL-Types representing the parameter list of the wanted method
     * @throws OclTypeException if the class does not contain a behavioural feature with the given signature, this exception is thrown
     */
    public Type navigateParameterized(String name, Type[] params) {
	String nameString = new String(name + "(");
	TJFeature f=null;
	Iterator it;
        LinkedList candidates = new LinkedList();

	//		System.out.println("navigateParameterized called, Parameter: " + "Name: "+name+", Parameter:" );
	if (params!=null){
	    for (int i=0; i<params.length; i++) {
		if (params[i] instanceof TJClassifier) {
		    if (i < params.length - 1) {
			nameString = nameString + ((TJClassifier)params[i]).getName() + ",";
		    } else {
			nameString = nameString + ((TJClassifier)params[i]).getName();
		    }
		} else {
                    String s = new String();
                    if (params[i] == Basic.INTEGER) {
                        s = "int";
                    } else if (params[i] == Basic.BOOLEAN) {
                        s = "boolean";
                    } else if (params[i] == Basic.REAL) {
                        s = "double";
                    } else if (params[i] == Basic.STRING) {
                        s = "java.lang.String";
                    }

		    if (i < params.length - 1) {
			nameString = nameString + s + ",";
		    } else {
			nameString = nameString + s;
		    }
		}
	    }
	}
	nameString = nameString + ")";
	//		System.out.println("  nameString: " + nameString);

	Type ret=null;

	//        System.out.println("    " + getName() + " has " + features.size() + " features:");
	//		System.out.println("    Possible Candidates are: ");

        it = features.keySet().iterator();
	while (it.hasNext()){
	    String key = it.next().toString();
	    //            System.out.println("  " + key);
	    if (key.indexOf('(') == -1) {
                //System.out.println("Why no ((((ing ( ??");
            } else {
		//		System.out.println("    Is it: " + key.substring(0,key.indexOf('(')) + "?");
		if (key.substring(0,key.indexOf('(')).equals(name)){
		    //					System.out.println("    Maybe, the name is identical");
		    candidates.addLast((TJFeature)features.get(key));
                }
		//f=(TJFeature)features.get(key);
	    }
	}
	//		if (f!=null && f instanceof TJBehaviouralFeature) {
	boolean fits = false;
	//        System.out.println("    There are " + candidates.size() + " candidates");
        if (candidates.size() > 0) {
	    /*			LinkedList candidatesRecoder = new LinkedList();
				recoder.java.declaration.TypeDeclaration ratThis = (recoder.java.declaration.TypeDeclaration)(recoderclasses.get(this.getName()));
				recoder.list.MethodList ml = ratThis.getAllMethods();
				recoder.abstraction.Method m;
				for (int i = 0; i < ml.size(); i++) {
				m = ml.getMethod(i);
				System.out.print(m.getName() + ", ");
				if (m.getName() == name) {
				candidatesRecoder.addLast(m);
				}
				}
	    */
            it = candidates.iterator();
	    while (it.hasNext()) {
		TJBehaviouralFeature tbf=(TJBehaviouralFeature)it.next();
		//		        System.out.println("    Comparing with candidate " + tbf.getName());
		Type[] featureParams=tbf.getParameters();
		fits = (params.length==featureParams.length);
		for (int i=0; fits && i<params.length; i++) {
		    if (!params[i].conformsTo(featureParams[i])) {
			fits=false;
                    }
		}
		if (fits) {
                    ret=tbf.getType();
		    //		    System.out.println("    Returning " + tbf.getName());
                    break;
                }
            }
	}
	if (ret==null) {
	    if (name.equals("oclIsKindOf") && params.length==1 && params[0] instanceof OclType) {
		ret=Basic.BOOLEAN;
	    } else if (name.equals("oclIsTypeOf") && params.length==1 && params[0] instanceof OclType) {
		ret=Basic.BOOLEAN;
	    } else if (name.equals("oclAsType") && params.length==1 && params[0] instanceof OclType) {
		ret=((OclType)params[0]).getType();
	    }
	}

	if (ret==null) {
            if (fits) {
                return new Any.VoidAny();
                //return ret;
            } else {
		throw new OclTypeException(this.name+" has no feature \""+name+"("+DefaultTypeFactory.toString(params)+")\"");
            }
	}
	return ret;
    }
    /** @return true if this type has a state with the given name
     */
    public boolean hasState(String stateName) {
	// 	System.out.println("  hasState aufgerufen");
	return false;
    }
    
    public String toString() {
	return name;
    }
    
    public boolean equals(TJClassifier o) {
	return (name == o.getName());
    }


}
