// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.logic.*;
import java.util.*;

/** class represents the result of a reading a proof obligation input. 
 * Usually, the result is just one term, but in some cases several terms are
 * created. This class offers access for both kinds of results.
*/
public class POInputResult {

    private List terms=new LinkedList();
    private List names=new LinkedList();

    /** creates a new result containing one term.
     */
    public POInputResult(String s, Term t) {
	terms.add(t);
	names.add(s);
    }

    /** creates a new empty result.
     */
    public POInputResult() {
    }

    /** returns the first term of this result
     */
    public Term term() {
	return (Term)terms.get(0);
    }

    /** returns the name of the first term of this result
     */
    public String name() {
	return (String)names.get(0);
    }

    /** returns the i th term of this result
     */
    public Term term(int i) {
	return (Term)terms.get(i);
    }

    /** returns the name of the i th term of this result
     */
    public String name(int i) {
	return (String)names.get(i);
    }

    /** returns the number of terms in this result container
     */
    public int size() {
	return terms.size();
    }

    /** adds a term to this result container
     */
    public void addTerm(String s, Term t) {
	terms.add(t);
	names.add(s);
    }

}
