// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/** this class encapsulates a pair of boolean and
 * SVInstantiations offering a possible instantiation. It is
 * created by matchFind of class Taclet or its subclasses */
package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.rule.inst.SVInstantiations;
public class MatchResult {
    /** Singleton for failed match */
    final static public MatchResult FAILED=new
	MatchResult(false, SVInstantiations.EMPTY_SVINSTANTIATIONS);
    // true iff match found
    private boolean successful;
    // contains necessary instantiations
    private SVInstantiations svInst;
    
    /** creates a MatchResult */
    private MatchResult(boolean successful, SVInstantiations svInst) {
	this.successful = successful;
	this.svInst = svInst;
    }

    /** creates a MatchResult. Representing a valid matching */
    public MatchResult(SVInstantiations svInst) {
	this(true, svInst);
    }
    
    /** returns the found instantiations
     * @return the found instantiations
     */
    public SVInstantiations instantiations() {
	return svInst;
    }
    
    /** returns true iff matching is valid
     * @return true iff matching is valid
     */
    public boolean success() {
	return successful;
    }

    /** toString */
    public String toString() {
	return "MatchResult was "+(!success() ? "not successful" :
				   "successful. Instantiations:"+instantiations());
    }
    
}
