// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/**
 * tests if match checks the variable conditions in Taclets. 
 */
package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.sort.*;
import junit.framework.*;
import de.uka.ilkd.key.proof.*;
import java.io.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.java.statement.Try;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.ArrayOfProgramElement;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.rule.inst.*;


public class TestMatchTaclet extends TestCase {

    FindTaclet if_addrule_conflict;
    FindTaclet find_addrule_conflict;
    FindTaclet count_var_occ_in_addrule;
    FindTaclet if_find_clash;
    FindTaclet if_add_no_clash;
    FindTaclet not_free_conflict;
    FindTaclet all_left;
    FindTaclet assign_n;
    TacletApp close_rule;
    Term matchExc;
    Taclet[] conflict = new Taclet[4];

    public TestMatchTaclet(String name) {
	super(name);
	//	Debug.ENABLE_DEBUG = false;
    }

    public void setUp() {	
	TacletForTests.setStandardFile("de/uka/ilkd/key/logic/testRuleMatch.txt");
	
	TacletForTests.parse();
	all_left = (FindTaclet)
		TacletForTests.getTaclet("TestMatchTaclet_for_all").taclet();
	if_addrule_conflict = (FindTaclet)
	    TacletForTests.getTaclet("if_addrule_clash").taclet();

	find_addrule_conflict = (FindTaclet)
	    TacletForTests.getTaclet("find_addrule_clash").taclet();
	
	count_var_occ_in_addrule = (FindTaclet)
	    TacletForTests.getTaclet("count_var_occ_in_addrule").taclet();
	
	if_find_clash  = (FindTaclet)
	    TacletForTests.getTaclet("if_find_clash").taclet();

	if_add_no_clash  = (FindTaclet)
	    TacletForTests.getTaclet("if_add_no_clash").taclet();

	not_free_conflict = (FindTaclet)
	    TacletForTests.getTaclet("not_free_conflict").taclet();
	close_rule = TacletForTests.getTaclet("close_rule");

	conflict[0] = (FindTaclet) 
	    TacletForTests.getTaclet("test_rule_one").taclet();
	conflict[1] = (FindTaclet) 
	    TacletForTests.getTaclet("test_rule_two").taclet();
	conflict[2] = (FindTaclet) 
	    TacletForTests.getTaclet("test_rule_three").taclet();
	conflict[3] = (FindTaclet) 
	    TacletForTests.getTaclet("test_rule_four").taclet();

	assign_n = (FindTaclet) 
	    TacletForTests.getTaclet("TestMatchTaclet_assign_n").taclet();


    }
    
    public void testStatementListMatch() {
	Term match = TacletForTests.parseTermNewJI("<{ l1:{l2:{while (true) {break; "
					    +"int k=1; {int j = 1; j++;} int c = 56;}}} }> true");
	FindTaclet break_while =  (FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_break_while").taclet();   
	SVInstantiations svi = (break_while.matchJavaProgram
	       (match.javaBlock(), 
		(ContextTerm)break_while.find().javaBlock().program()
		, break_while.createInitialInstantiation(), null)); 	
	assert("Matches have been expected.",svi != null);
	SchemaVariable sv = TacletForTests.svLookup("#stmnt_list");
	assert("Expected list of statement to be instantiated.",
	       svi.isInstantiated(sv));
	assert("The three statements behind the break should be matched.",
	       ((ArrayOfProgramElement)svi.getInstantiation(sv)).size() == 3);
    }

    public void testProgramMatch0() {
	Term match = TacletForTests.parseTerm("<{ l1:{l2:{while (true) {break;} "
					    +"int k=1;}} }> true");
	FindTaclet taclet=(FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_whileright").taclet();   
	SVInstantiations svi = (taclet.matchJavaProgram
	       (match.javaBlock(), 
		(ContextTerm)taclet.find().javaBlock().program()
		, taclet.createInitialInstantiation(), null)); 

	assert("There should be instantiations",svi!=null);
	assert("#e2 should be instantiated",
	       svi.isInstantiated(TacletForTests
				  .svLookup("#e2")));
	assert("#p1 should be instantiated",
	       svi.isInstantiated(TacletForTests
				  .svLookup("#p1")));
	
	Term matchTwo=TacletForTests.parseTerm("<{ l1:{l2:{while (true) {boolean b=true; break;} "
					    +"}int k=1;} }> true");
	FindTaclet tacletTwo=(FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_whileright_labeled").taclet(); 
	
	svi = (tacletTwo.matchJavaProgram
	     (matchTwo.javaBlock(), 
	      (ContextTerm)tacletTwo.find().javaBlock().program(), 
	      tacletTwo.createInitialInstantiation(), null)); 
	assert(svi != null);

	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#e2")));
	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#p1")));
	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#lab")));

	Term match3 = TacletForTests.parseTerm("<{ l1:{l2:{while (true) {boolean b=false; break;} "
					    +"int k=1;}} }> true");
	FindTaclet taclet3=(FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_whileright_labeled").taclet(); 
	
	svi=(taclet3.matchJavaProgram
	     (match3.javaBlock(), 
	      (ContextTerm)taclet3.find().javaBlock().program(), 
	      taclet3.createInitialInstantiation(), null)); 
	assert(svi==null);

	Term emptyBlock = 
	    TacletForTests.parseTerm("<{ { {} int i = 0; } }> true");
	FindTaclet empty_block_taclet=(FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_empty_block").taclet(); 
	
 	svi = (empty_block_taclet.matchJavaProgram
 	     (emptyBlock.javaBlock(), 
 	      (ContextTerm)empty_block_taclet.find().
 	      javaBlock().program(), 
 	      empty_block_taclet.createInitialInstantiation(), null)); 
 	assert(svi != null);

	Term emptyBlock2 = 
	    TacletForTests.parseTerm("<{ { {} } }> true");
	svi = (empty_block_taclet.matchJavaProgram
	     (emptyBlock2.javaBlock(), 
	      (ContextTerm)empty_block_taclet.find().
	      javaBlock().program(), 
	      empty_block_taclet.createInitialInstantiation(),null)); 
	assert(svi != null);

	Debug.out("%%%%%%%%%%%%");
	Term emptyBlock3 = 
	    TacletForTests.parseTerm("<{ { {} l1:{} } }> true");
	svi = (empty_block_taclet.matchJavaProgram
	     (emptyBlock3.javaBlock(), 
	      (ContextTerm)empty_block_taclet.find().
	      javaBlock().program(), 
	      empty_block_taclet.createInitialInstantiation(), null)); 
	assert(svi != null);


	FindTaclet var_decl_taclet=(FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_variable_declaration").taclet(); 
	svi = (var_decl_taclet.matchJavaProgram
	     (emptyBlock.javaBlock(), 
	      (ContextTerm)var_decl_taclet.find().
	      javaBlock().program(), 
	      var_decl_taclet.createInitialInstantiation(), null)); 	
	assert(svi == null);	

	Term emptyLabel = 
	    TacletForTests.parseTerm("<{ { l1:{} } }> true");
	FindTaclet empty_label_taclet=(FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_empty_label").taclet(); 
	svi = (empty_label_taclet.matchJavaProgram
	     (emptyLabel.javaBlock(), 
	      (ContextTerm)empty_label_taclet.find().
	      javaBlock().program(), 
	      empty_label_taclet.createInitialInstantiation(), null)); 
	assert(svi != null);

	Term emptyLabel2 = 
	    TacletForTests.parseTerm("<{ l2:{ l1:{} } }> true");
	svi = (empty_label_taclet.matchJavaProgram
	     (emptyLabel2.javaBlock(), 
	      (ContextTerm)empty_label_taclet.find().
	      javaBlock().program(), 
	      empty_label_taclet.createInitialInstantiation(), null)); 
	assert(svi != null);

	Term emptyLabel3 = 
	    TacletForTests.parseTerm("<{ {l3:{{l2:{l1:{}}}} int i = 0;} }> true");
	svi = (empty_label_taclet.matchJavaProgram
	     (emptyLabel3.javaBlock(), 
	      (ContextTerm)empty_label_taclet.find().
	      javaBlock().program(), 
	      empty_label_taclet.createInitialInstantiation(), null)); 
	assert(svi != null);




    }

        
        public void testProgramMatch1() {
	de.uka.ilkd.key.java.Recoder2KeY c2k
	    =new de.uka.ilkd.key.java.Recoder2KeY
		("/usr/lib/java/jre/lib/rt.jar", TacletForTests.javaInfo().rec2key(),
		 TacletForTests.javaInfo().getTypeConverter());
	JavaBlock jb=c2k.readBlock("{ int i; int j; i=++j;"
				   +" while(true) {break;}}", 
				   c2k.createEmptyContext());

	de.uka.ilkd.key.java.StatementBlock sb
	    =(de.uka.ilkd.key.java.StatementBlock)jb.program();

	JavaBlock match = JavaBlock.createJavaBlock
	    (new de.uka.ilkd.key.java.StatementBlock
		(new de.uka.ilkd.key.java.ArrayOfStatement
		    (new de.uka.ilkd.key.java.Statement[]{
			(de.uka.ilkd.key.java.Statement)sb.getChildAt(2),
			(de.uka.ilkd.key.java.Statement)sb.getChildAt(3)
		    })));

	FindTaclet taclet=(FindTaclet)TacletForTests
	    .getTaclet("TestMatchTaclet_preincrement").taclet();   
	SVInstantiations svi=(taclet.matchJavaProgram
			      (match, (ContextTerm)taclet.find().javaBlock()
			       .program(), taclet.createInitialInstantiation(), null)); 

	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#slhs1")));
	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#slhs2")));
    }
    
    public void testProgramMatch2() {
	Term match = TacletForTests.parseTermNewJI("<{int i; int k;}>(<{for (int i=0;"
					      +" i<2; i++) {break;} "
					      +"int k=1; }> true)");
	FindTaclet taclet
	    =(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_for_right").taclet();   
	SVInstantiations svi=(taclet.matchJavaProgram
			   (match.sub(0).javaBlock(), 
			    (ContextTerm)taclet.find().javaBlock().program(), 
			    taclet.createInitialInstantiation(), null)); 

	assert(svi!=null);
	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#e2")));
	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#p1")));
	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#i")));
	assert(svi.isInstantiated(TacletForTests
				  .svLookup("#sl")));	
    }
    /*
    public void testProgramMatch3() {
	Term match = TacletForTests.parseTerm("<{ {int i=0; break; }  }> true");
	FindTaclet taclet
	    =(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_local_variable_rename").taclet();   
	SVInstantiations svi=(taclet.matchJavaProgram
			   (match.javaBlock(), 
			    (ContextTerm)taclet.find().javaBlock().program(), 
			    taclet.createInitialInstantiation())); 

	System.out.println(svi);
    }
    */
        
    public void testProgramMatch4() {
	Term match = TacletForTests.parseTerm("<{{while (1==1) "
					    +"{if (1==2) {break;}} "
					    +"return 1==3;}}>A");

	FindTaclet taclet
	    =(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_while0").taclet();   
	SVInstantiations svi=(taclet.match
			      (match, 
			       taclet.find(), 
			       taclet.createInitialInstantiation(), null)); 
	assert(svi!=null);
	//	System.out.println(svi);
    }


    // Test currently disabled. Broken because of change in TacletForTests
    public void xtestProgramMatchTry() {
	Term match = TacletForTests.parseTerm
	    ("<{Exception f; try{ throw new Exception(); while (1==1) {if (1==2) {break;}} return 1==3; int i=17; } catch (Exception e) { return null;}}>A");	
	Try tr=(Try)((StatementBlock)match.javaBlock().program()).getChildAt(1);
	JavaBlock jb=JavaBlock.createJavaBlock(new StatementBlock(tr));
	FindTaclet taclet
	    =(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_try_throw_to_if").taclet();   
	SVInstantiations svi=(taclet.matchJavaProgram
			      (jb, 
			       (ContextTerm)taclet.find().javaBlock().program(), 
			       taclet.createInitialInstantiation(), null)); 
	assert(svi!=null);
	//-----
	match = TacletForTests.parseTerm("<{Exception e; try{ throw e; ; ;} catch(Exception e1) {int c = 0;} catch(RuntimeException e2) {int b = 0;} finally{int a = 1;}}>true");
	tr=(Try)((StatementBlock)match.javaBlock().program()).getChildAt(1);
	jb=JavaBlock.createJavaBlock(new StatementBlock(tr));
	taclet
	    =(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_empty_try").taclet();   
	svi=(taclet.matchJavaProgram(jb, 
			       (ContextTerm)taclet.find().javaBlock().program(), 
			       taclet.createInitialInstantiation(), null)); 
	//	assert(svi!=null);
	//-----
	match = TacletForTests.parseTerm("<{Exception e; try{ } catch(Exception e1) {int c = 0;} catch(RuntimeException e2) {int b = 0;} finally{int a = 1;}}>true");
	tr=(Try)((StatementBlock)match.javaBlock().program()).getChildAt(1);
	jb=JavaBlock.createJavaBlock(new StatementBlock(tr));
	taclet
	    =(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_empty_try").taclet();   
	svi=(taclet.matchJavaProgram
			      (jb, 
			       (ContextTerm)taclet.find().javaBlock().program(), 
			       taclet.createInitialInstantiation(), null)); 
	assert(svi!=null);
    }
    
    
        
    public void testVarOccursInIfAndAddRule() {

	Term match = TacletForTests.parseTerm("all z:testSort.(p(z) -> A)");

	// check some occurrences
	SchemaVariable forSV_a = (SchemaVariable)TacletForTests.getVariables().
	    lookup(new Name("a"));
 	assert("a is not found in the if part of rule if_addrule_conflict", 
 	       if_addrule_conflict.lookupInAreaOne(forSV_a));
 	assert("a is not found in an addrule of rule "+
 	       "if_addrule_conflict", 
	       if_addrule_conflict.lookupInAddRulesArea(forSV_a, true));
 	assert("a occurs twice but rule say instantiationOccurrence"+
 	       "correct", 
	       !if_addrule_conflict.correctInstantiationAreaOccurrence(forSV_a));

	// test at the subformula p(z) -> A that has a free variable
	// therefore no match should be found

	Sequent seq = Sequent.createSequent
	    (Semisequent.EMPTY_SEMISEQUENT.insert(0, new 
						  ConstrainedFormula(match.sub(1), 
			       Constraint.BOTTOM)), Semisequent.EMPTY_SEMISEQUENT);

	assert("An area conflict should happen because there is a free"+
	       " variable and the matching part is in the if and addrule", 
	       if_addrule_conflict.matchIf
	       (seq,
		if_addrule_conflict.createInitialInstantiation(), null).size() == 0);
	       
 	// we bind the free variable now a match should be found
	seq = Sequent.createSequent
	    (Semisequent.EMPTY_SEMISEQUENT.insert(0, new ConstrainedFormula
						  (match, 
						   Constraint.BOTTOM)), 
	     Semisequent.EMPTY_SEMISEQUENT );

	assert("No area conflict should happen because all variables are bound.", 
	       if_addrule_conflict.matchIf
	       (seq, 
		if_addrule_conflict.createInitialInstantiation(), null).size() != 0);
    }


    public void testVarOccursInFindAndAddRule() {
	Term match = TacletForTests.parseTerm("all z:testSort.(p(z) -> A)");

	// check some occurrences
	SchemaVariable forSV_a = (SchemaVariable)TacletForTests.getVariables().
	    lookup(new Name("a"));

	assert("a is not found in the find part of rule find_addrule_conflict", 
	       find_addrule_conflict.lookupInTerm(forSV_a, find_addrule_conflict.find())); 
	assert("a is not found in an addrule of rule find_addrule_conflict", 
	       find_addrule_conflict.lookupInAddRulesArea(forSV_a, true)); 
	assert("a occurs twice but rule say instantiationOccurrence"+
	       "correct", !find_addrule_conflict.correctInstantiationAreaOccurrence(forSV_a)); 
	//seq contains term that can match but has a free variable, so
	//matching to a should be not possible
	assert("A match has been found but there is a free variable in"+
	       " the term that has been matched and therefore an area"+
	       " conflict with find and addrule should have happened.",
	       find_addrule_conflict.match(match.sub(0), 
					   find_addrule_conflict.find(), false,
					   find_addrule_conflict.createInitialInstantiation(), 
					   null) == null);

	// var is not free, match should be found 
	assert("A match should have been found,"+
	       " because here there formerly free variable is bound.", 
	       find_addrule_conflict.match(match, find_addrule_conflict.find(), false, 
			   find_addrule_conflict.createInitialInstantiation(), null) != null);	           
    }


    public void testVarOccursInTwoAddRules() {
	// tests if the occurrence of schemavariables in addrules is
	// count in the correct way
	// check some occurrences
	SchemaVariable forSV_a = (SchemaVariable)TacletForTests.getVariables().
	    lookup(new Name("a"));

	assert("a is not found in the find part of rule"+
	       "count_var_occ_in_addrule", !
	       (count_var_occ_in_addrule.
		lookupInTerm(forSV_a, 
			     count_var_occ_in_addrule.find()) ||
		count_var_occ_in_addrule.lookupInAreaOne(forSV_a)));  
	assert("a is found less than twice in an addrule of rule "+
	       "'count_var_occ_in_addrule'", 
	       count_var_occ_in_addrule.lookupInAddRulesArea(forSV_a, false)); 

	assert("a occurs twice but rule thinks the instantiationOccurrence"+
	       " is correct", 
	       !count_var_occ_in_addrule.correctInstantiationAreaOccurrence(forSV_a)); 
    }

    public void testRWVarOccursFindAndIf() {
	// the find expression is not a sequent, therefore find and if
	// are two different areas and if find matches a term that
	// contains a free variable, no match should be possible  
	//seq contains term that can match but has a free variable, so
	//matching to a should be not possible
	Term match = TacletForTests.parseTerm("all z:testSort.(p(z) -> A)");

	assert("Match found but match term contains free var and"+
	       "matching var occurs in two instantiation areas"+
	       " (if and find)", 
	       if_find_clash.match(match.sub(0), if_find_clash.find(), false, 
			   if_find_clash.createInitialInstantiation(), null) == null); 


	assert("Match not found", 
	       if_find_clash.match(match, if_find_clash.find(), false,
			   if_find_clash.createInitialInstantiation(), null) != null);	           
    }

    public void testRWVarOccursInAddAndIf() {
	// no clash should happen because in this case the add and if
	// sections are the same area
	Term match = TacletForTests.parseTerm("all z:testSort.(p(z) -> A)");

	assert("Match not found but should exist"+
	       " because add and if are same area",
	       if_add_no_clash.match(match.sub(0), if_add_no_clash.find(), false,
		     if_add_no_clash.createInitialInstantiation(), null) != null); 
    }
    

    
    public void testXNotFreeInYConflict() {
	Term free_in = TacletForTests.parseTerm("all z:testSort.(p(z) & p(f(z)))");
	// matching the not_free_conflict Taclet with (P(f(z),z) should
	// result in a conflict, because z is free in f(z) but
	// the Taclet demands z not free in f(z)
	assert("Match should not be found because of conflict with "+
	       "..not free in..", NoPosTacletApp.createNoPosTacletApp
	       (not_free_conflict,
		not_free_conflict.match
		(free_in, not_free_conflict.find(), false,
		 not_free_conflict.createInitialInstantiation(), null)) == null);

	Term not_free_in = TacletForTests.parseTerm("all z:testSort.(p(z) & p(c))");
 	assert("Match should be found because .. not free in.. "+
 	       "is not relevant", NoPosTacletApp.createNoPosTacletApp
	       (not_free_conflict,
		not_free_conflict.match
		(not_free_in, not_free_conflict.find(), false,
		 not_free_conflict.createInitialInstantiation(), null)) != null);
    }


    private boolean isRuleIn(ListOfNoPosTacletApp l, Taclet rule) {
	IteratorOfNoPosTacletApp it=l.iterator();
	while(it.hasNext()) {
	    if (it.next().taclet()==rule) return true;
	}
	return false;
    }

    public void testCloseWithBoundRenaming() {
	Term closeable_one = TacletForTests.parseTerm("all z:testSort.p(z)");
	Term closeable_two = TacletForTests.parseTerm("all y:testSort.p(y)");
	Sequent seq = Sequent.createSequent
	    (Semisequent.EMPTY_SEMISEQUENT.insert
	     (0, new ConstrainedFormula(closeable_one)), 
	     Semisequent.EMPTY_SEMISEQUENT.insert
	     (0, new ConstrainedFormula(closeable_two))); 	
	TacletIndex index = new TacletIndex();
	index.add(close_rule.taclet());
	TacletApp tacletApp = index.getSuccedentTaclet(true, closeable_two,
	     SLListOfHeuristic.EMPTY_LIST, null).iterator().next();
	assert("Match should be possible(modulo renaming)",
	       tacletApp.taclet().matchIf(seq, 
				      tacletApp.instantiations(), null).size()>0);
    }
   
    // a greater test 
    public void testConflict() {
	Term match = TacletForTests.parseTerm("p1(m1(n))");
	for (int i=0; i<conflict.length; i++) {
	    assert("Match should not be found because of area conflict:"+i,
		   conflict[i].match
		   (match, ((FindTaclet)conflict[i]).find(), 
		    conflict[i].createInitialInstantiation(), null)==null);	    
	}
    }
    
    // test match of update terms
    public void testUpdateMatch() {

	Term match = TacletForTests.parseTerm("<{int i;}>{i:=2(#)}(all z:nat.(q1(z)))");
	match = match.sub(0);
	assert("Instantiations should be found as updates can be ignored if "+
	       "only the term that is matched has an update and the "+
	       "template it is matched to has none.",
	       all_left.match(match, ((FindTaclet)all_left).find(), 
		    true, all_left.createInitialInstantiation(), null)!=null);
		
	Term match2 = TacletForTests.parseTerm("<{int i;}>{i:=2(#)} true");
	match2 = match2.sub(0);
	assert("Instantiations should be found.",
	       assign_n.match(match2, ((FindTaclet)assign_n).find(), 
			      true, assign_n.createInitialInstantiation(), null)!=null);
    }


    public void testProgramMatchEmptyBlock() {
	Term match = TacletForTests.parseTerm("<{ }>true ");
	FindTaclet taclet
	    =(FindTaclet)TacletForTests.getTaclet("empty_diamond").taclet();   
	SVInstantiations svi=(taclet.match
			      (match, 
			       taclet.find(), 
			       taclet.createInitialInstantiation(), null)); 

	assert(svi!=null);

	match = TacletForTests.parseTerm("<{ {} }>true ");
	taclet=(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_empty_block").taclet();   

	svi=(taclet.match(match, 
			taclet.find(), 
			taclet.createInitialInstantiation(), null)); 

	assert(svi!=null);

	match = TacletForTests.parseTerm("<{ {int i = 0;} }>true ");
	svi=(taclet.match(match, 
			taclet.find(), taclet.createInitialInstantiation(), null)); 

	assert("The block is not empty",svi==null);

    }

    public void testWithSubSorts() {
	Sort osort1=(Sort)TacletForTests.getSorts().lookup(new Name("Obj"));
	Sort osort2=new ObjectSort(new Name("os2"), osort1);
	Sort osort3=new ObjectSort(new Name("os3"), osort1);
	Sort osort4=new ObjectSort(new Name("os4"), ObjectSort.EMPTY_SORT_SET
				   .add(osort2).add(osort3));
	Function v4=new Function(new Name("v4"), osort4, new Sort[0]);	
	TermFactory tf=TermFactory.DEFAULT;
	Term match=tf.createFunctionTerm(v4);
	FindTaclet taclet=(FindTaclet)TacletForTests.getTaclet
	    ("TestMatchTaclet_subsort").taclet();   
	SVInstantiations svi=(taclet.match(match, 
					 taclet.find(), 
					 taclet.createInitialInstantiation(), 
					 null)); 
	assert(svi==null);
    }

    public void testNoContextMatching() {
	Term match = TacletForTests.parseTerm("<{{ int i = 0;}}>true ");
	FindTaclet taclet=(FindTaclet)TacletForTests.getTaclet
	    ("TestMatchTaclet_nocontext").taclet();   
	SVInstantiations svi=(taclet.match(match, 
					 taclet.find(), 
					 taclet.createInitialInstantiation(), 
					 null)); 
	assert("No context matching corrupt.",svi!=null);
    }


    public void testBugsThathaveBeenRemoved() {

	Term match = TacletForTests.parseTerm("<{ int i = 0; }>true ");
	FindTaclet taclet=(FindTaclet)TacletForTests.getTaclet
	    ("TestMatchTaclet_eliminate_variable_declaration").taclet();   
	SVInstantiations svi=(taclet.match(match, 
					 taclet.find(), 
					 taclet.createInitialInstantiation(), 
					 null)); 

	assert("The reason for this bug was related to the introduction of "+
	       "statementlist schemavariables and that we could not end the "+ 
	       "match if the size of nonterminalelements was unequal "+
	       "The solution was to weaken the check for statement blocks but NOT "+
	       "for other statement or expressions containers. The bug occured because "+
	       "the weaker test was also "+
	       "performed for expressions.", svi==null);

	match = TacletForTests.parseTerm("<{ {{throw null;} int i = 0;} }>true ");
	taclet=(FindTaclet)TacletForTests.getTaclet("TestMatchTaclet_throw_in_block").taclet();   
	svi=(taclet.match(match, 
			taclet.find(), 
			taclet.createInitialInstantiation(), null)); 
	assert("No match expected.", svi == null);

    }

    public static void main(String ags[]) {
	TestMatchTaclet t=new TestMatchTaclet("XXX");
	t.setUp();
	t.xtestProgramMatchTry();
    }
    
}
