// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.op.UpdateOp;
import de.uka.ilkd.key.logic.Term;

/** this class represents a pair of an update operator and the term
 * that is used to update the variable(attribute) of update operator
 */

public class UpdatePair {

    /** the update operator */
    private final UpdateOp op;

    /** the term that is used to update the variable or attribute */
    private final Term term;


    /** creates a tupel that consists of the update operator and the
     * associated term that describes the state of the
     * variable/attribute of the op
     */
    public UpdatePair(UpdateOp op, Term term) {
	this.op   = op;
	this.term = term;
    }
    
    public UpdateOp updateOp() {
	return op;
    }

    public Term update() {
	return term;
    }

    public boolean equals(Object o) {
	if (!(o instanceof UpdatePair)) {
	    return false;
	}
	UpdatePair cmp = (UpdatePair) o;
	return op.equals(cmp.op) && term.equals(cmp.term);
    }

    public String toString() {
	return "("+op.variable()+","+term+")";
    }
}
