// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.metaconstruct;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.statement.Try;
import de.uka.ilkd.key.java.statement.Finally;
import de.uka.ilkd.key.java.ArrayOfStatement;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
/** This class is used to perform program transformations needed 
 * for the symbolic execution of an empty try statement as 
 * try {} beta ; where beta is the catch-finally part
 */
public class EmptyTryToIf extends ProgramMetaConstruct {

     /** creates a empty-try-to-if ProgramMetaConstruct 
     * @param emptyTry the Statement contained by 
     * the meta construct 
     */
    public EmptyTryToIf(Statement emptyTry) {
	super("empty-try", emptyTry); 
    }

    /** performs the program transformation needed for symbolic
     * program transformation 
     * @param javaInfo the JavaInfo with all necessary information 
     * about the java programs
     * @return the transformated program
     */
    public ProgramElement symbolicExecution(ProgramElement pe,
					    JavaInfo javaInfo,
					    SVInstantiations insts) {
	Try try_stmnt = (Try) pe;
	for (int i = 0; i < try_stmnt.getBranchCount(); i++) {
	    if (try_stmnt.getBranchAt(i) instanceof Finally) {
		return ((Finally)try_stmnt.getBranchAt(i)).getBody();
	    }
	}
	return new StatementBlock(new ArrayOfStatement(new Statement[0]));
    }
}
