// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.util.install;

import java.io.File;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/** The installer frame consists of a header, the content pane and a
 * button panel. This content pane is an instance of this class.
 */
public class LibrarySettingsPane extends InstallationPane {

    private String[] libnames =new String[]{
	"antlr.jar", "dresden-ocl-demo.jar", "xerces-1_4_4.jar", 
	"recoder.jar", "xml.jar"};
    
    private InstallationPathChooser[] chooser=
	new InstallationPathChooser[libnames.length];

    public LibrarySettingsPane() {
	super("Library");
	setup();
    }
    
    private void setup() {
	setLayout(new BorderLayout());
	JTextArea text=new JTextArea(5, 30);
	text.setLineWrap(true);
	text.setWrapStyleWord(true);
	text.setText("The libraries below are required by KeY."+
		     "You have to install the libraries first, therefore they can be "+
		     "downloaded from the KeY-Homepage http://i12www.ira.uka.de/~key/");
	text.setEditable(false);
	text.setBackground(getBackground());
	add(text, BorderLayout.NORTH);

 	Box entries=Box.createVerticalBox();
	for (int i=0; i<libnames.length; i++) {
	    chooser[i]=new InstallationPathChooser
		(libnames[i], 
		 System.getProperty("user.dir")+File.separator+libnames[i], 
		 JFileChooser.FILES_ONLY);
	    entries.add(chooser[i]);
	    if (i==0) {
		JPanel checkBoxPanel=new JPanel();
		checkBoxPanel.setAlignmentY(SwingConstants.TOP);
		checkBoxPanel.add(createCheckBox());
		add(checkBoxPanel, BorderLayout.EAST);		
	    }
	} 
	add(entries, BorderLayout.CENTER);	
    }

    private JCheckBox createCheckBox() {
	JCheckBox box=new JCheckBox("all", false);
	box.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    if (((JCheckBox)ce.getSource()).isSelected()) {
			for (int j=1; j<libnames.length; j++) {
			    File file=new File(chooser[0].getPath());
			    if (file.isDirectory()) {
				chooser[j].setPath(file.getAbsolutePath()+
						   File.separator+libnames[j]);				       
			    } else {
				String path=file.getAbsolutePath().
				    substring(0, file.getAbsolutePath().
					      lastIndexOf(File.separatorChar));
				chooser[j].setPath(path+File.separator+libnames[j]);	  
			    }
			    chooser[j].setEnabled(false);
			}
		    } else {
			for (int j=1; j<libnames.length; j++) {
			    chooser[j].setEnabled(true);
			}
		    }
		}
	    });		
	return box;
    }

    public String path(int index) {
	return chooser[index].getPath();
    }


}
